/*
 * ==========================================================================
 *
 *       Filename:  bex.h
 *
 *    Description:  header for bex (baltam executable)
 *
 *        Version:  1.0
 *        Created:  06/18/2021 10:29:39 AM
 *       Revision:  01/10/2022 10:23    AM
 *       Compiler:  gcc
 *
 *         Author:  Haoyang Liu (@liuhy), liuhaoyang@pku.edu.cn
 *   Organization:  BICMR, PKU
 *
 * ==========================================================================
 */

#ifndef BALTAM_BEX_H
#define BALTAM_BEX_H

// 默认整数类型
#ifndef BALTAM_DEFAULT_INT
#include <stddef.h>
#define BALTAM_DEFAULT_INT ptrdiff_t
#endif

#include <stdint.h>
#include <stdbool.h>
#include "api_macro.h"
#include "bex/bx_op.h"
#include "bex/extern_fcn_type.h"

// 宏展开：统一工具箱接口
#define BALTAM_PLUGIN_FCN(word) \
    void word(int nlhs, bxArray *plhs[], int nrhs, const bxArray *prhs[])


#ifdef __cplusplus
extern "C" {
#endif

// 定义数据类型
typedef BALTAM_DEFAULT_INT baInt;
typedef baInt baIndex;
typedef baInt baSize;
typedef baIndex baSparseIndex;

// 定义用于传递内核变量的 C 结构体（不透明对象）
struct __bxArray;
typedef struct __bxArray bxArray;

typedef enum {
    bxREAL = 0,
    bxCOMPLEX
} bxComplexity;

typedef enum {
    bxUNKNOWN_CLASS = 0,
    bxINT_CLASS = 1, // 需要从 1 开始
    bxINT64_CLASS,
    bxDOUBLE_CLASS,
    bxSINGLE_CLASS,
    bxCHAR_CLASS = 7,
    bxLOGICAL_CLASS,
    bxSTRUCT_CLASS = 10,
    bxSTRING_CLASS = 11,
    bxEXTERN_CLASS = 15,
    bxCELL_CLASS = 19
} bxClassID;

// 函数句柄类型枚举类
typedef enum {
    bxFH_UNKNOWN = -1,
    bxFH_ANONYMOUS = 0, // 匿名函数
    bxFH_VARIABLE,      // 变量
    bxFH_BUILTIN,       // 内置函数
    bxFH_MFUNCTION,     // M 函数
    bxFH_SCRIPT         // 脚本
} bxFHandleType;

// bex 接口函数原型
// 获取数据
BALTAM_API bxClassID bxGetClassID(const bxArray*);
BALTAM_API const char * bxTypeCStr(const bxArray*);
BALTAM_API baSize bxGetM(const bxArray*);
BALTAM_API baSize bxGetN(const bxArray*);
BALTAM_API baSize bxGetStringLen(const bxArray*);
BALTAM_API baSize bxGetStringLength(const bxArray*, baIndex);
BALTAM_API double* bxGetPr(const bxArray*);
BALTAM_API int* bxGetInt32s(const bxArray*);
BALTAM_API int64_t* bxGetInt64s(const bxArray*);
BALTAM_API double* bxGetDoubles(const bxArray*);
BALTAM_API float* bxGetSingles(const bxArray*);
BALTAM_API void* bxGetComplexDoubles(const bxArray*);
BALTAM_API void* bxGetComplexSingles(const bxArray*);
BALTAM_API char* bxGetChars(const bxArray*);
BALTAM_API bool* bxGetLogicals(const bxArray*);
BALTAM_API const char* bxGetStringDataPr(const bxArray*);
BALTAM_API bxArray* bxGetField(const bxArray*, baIndex, const char *);
BALTAM_API baSize bxGetNumberOfFields(const bxArray*);
BALTAM_API bxArray* bxGetCell(const bxArray*, baIndex);
BALTAM_API const char* bxGetString(const bxArray*, baIndex);
BALTAM_API baInt bxAsInt(const bxArray*, int *);
BALTAM_API int bxAsCStr(const bxArray *, char *, baSize);
BALTAM_API bool bxIsInt32(const bxArray*);
BALTAM_API bool bxIsInt64(const bxArray*);
BALTAM_API bool bxIsDouble(const bxArray*);
BALTAM_API bool bxIsSingle(const bxArray*);
BALTAM_API bool bxIsComplex(const bxArray*);
BALTAM_API bool bxIsChar(const bxArray*);
BALTAM_API bool bxIsLogical(const bxArray*);
BALTAM_API bool bxIsString(const bxArray*);
BALTAM_API bool bxIsStruct(const bxArray*);
BALTAM_API bool bxIsCell(const bxArray*);
BALTAM_API bool bxIsRealDouble(const bxArray*);
BALTAM_API bool bxIsRealSingle(const bxArray*);
BALTAM_API bool bxIsComplexDouble(const bxArray*);
BALTAM_API bool bxIsComplexSingle(const bxArray*);
// 创建和删除
BALTAM_API bxArray* bxCreateDoubleMatrix(baSize, baSize, bxComplexity);
BALTAM_API bxArray* bxCreateNumericMatrix(baSize, baSize, bxClassID, bxComplexity);
BALTAM_API bxArray* bxCreateLogicalMatrix(baSize, baSize);
BALTAM_API bxArray* bxCreateCharMatrixFromStrings(baSize, const char **);
BALTAM_API bxArray* bxCreateString(const char *);
BALTAM_API bxArray* bxCreateStringObj(const char *);
BALTAM_API bxArray* bxCreateStructMatrix(baSize, baSize, int, const char **);
BALTAM_API bxArray* bxCreateCellMatrix(baSize, baSize);
BALTAM_API bxArray* bxCreateStringMatrix(baSize, baSize);
BALTAM_API bxArray* bxCreateStringMatrixFromStrings(baSize, baSize, const char **);
BALTAM_API bxArray* bxCreateInt32Scalar(int32_t);
BALTAM_API bxArray* bxCreateInt64Scalar(int64_t);
BALTAM_API bxArray* bxCreateDoubleScalar(double);
BALTAM_API bxArray* bxCreateSingleScalar(float);
BALTAM_API bxArray* bxCreateComplexDoubleScalar(double, double);
BALTAM_API bxArray* bxCreateComplexSingleScalar(float, float);
// BALTAM_API bxArray *bxCreateCharScalar(char);
BALTAM_API bxArray* bxCreateLogicalScalar(bool);
BALTAM_API bxArray* bxCreateStringScalar(const char *);
BALTAM_API bxArray* bxDuplicateArray(const bxArray*);
BALTAM_API bxArray* bxDuplicateArrayS(const bxArray*);
BALTAM_API void bxDestroyArray(bxArray*);
// 修改数据
BALTAM_API void bxSetM(bxArray*, baSize);
BALTAM_API void bxSetN(bxArray*, baSize);
BALTAM_API void bxResize(bxArray*, baSize, baSize);
BALTAM_API void bxSetStringFromCStr(bxArray*, const char *);
BALTAM_API void bxSetField(bxArray*, baIndex, const char *, bxArray*);
BALTAM_API void bxRemoveField(bxArray*, const char *);
BALTAM_API void bxSetCell(bxArray*, baIndex, bxArray*);
BALTAM_API void bxSetString(bxArray*, baIndex, const char *);

// 稀疏矩阵
BALTAM_API bxArray* bxCreateSparse(baSize, baSize, baSize, bxComplexity);
BALTAM_API bxArray* bxCreateSparseNumericMatrix(baSize, baSize, baSize, bxClassID, bxComplexity);
BALTAM_API bxArray* bxCreateSparseLogicalMatrix(baSize, baSize, baSize);
BALTAM_API bool bxIsSparse(const bxArray*);
BALTAM_API bool bxIsSparseRealDouble(const bxArray*);
BALTAM_API bool bxIsSparseRealSingle(const bxArray*);
BALTAM_API bool bxIsSparseComplexDouble(const bxArray*);
BALTAM_API bool bxIsSparseComplexSingle(const bxArray*);
BALTAM_API bool bxIsSparseLogical(const bxArray*);
BALTAM_API double* bxGetSparseDoubles(const bxArray*);
BALTAM_API float* bxGetSparseSingles(const bxArray*);
BALTAM_API void* bxGetSparseComplexDoubles(const bxArray*);
BALTAM_API void* bxGetSparseComplexSingles(const bxArray*);
BALTAM_API bool* bxGetSparseLogicals(const bxArray*);
BALTAM_API baSparseIndex* bxGetIr(const bxArray*);
BALTAM_API baSparseIndex* bxGetJc(const bxArray*);
BALTAM_API baSize bxGetNnz(const bxArray*);
BALTAM_API baSize bxGetNzmax(const bxArray*);
BALTAM_API void bxSetNzmax(bxArray*, baSize);
BALTAM_API void bxSparseFinalize(bxArray*);

// 函数句柄
BALTAM_API bool bxIsFunctionHandle(const bxArray*);
BALTAM_API bxFHandleType bxGetFunctionHandleType(const bxArray*);
BALTAM_API const char * bxGetFunctionHandleData(const bxArray*);

// 辅助函数
BALTAM_API int bxPrintf(const char *, ...);
BALTAM_API void bxErrMsgTxt(const char *);
BALTAM_API baIndex bxCalcSingleSubscript(const bxArray*, int, baIndex*);

// bex 外部数据
BALTAM_API int bxRegisterCStruct(const char *, cstruct_copy_t, cstruct_delete_t);
BALTAM_API void * bxGetCStruct(int, const bxArray*);
BALTAM_API bxArray *bxCreateCStruct(int, void *);
BALTAM_API int bxRegisterUnaryOperator(const char *, const char *, int, bexfun_t);
BALTAM_API int bxRegisterBinaryOperator(const char *, const char *, int, int, bexfun_t);
BALTAM_API int bxRegisterTernaryOperator(const char *, const char *, int, int, int, bexfun_t);
BALTAM_API int bxRegisterUnaryOperatorID(const char *, bxOperatorID, int, bexfun_t);
BALTAM_API int bxRegisterBinaryOperatorID(const char *, bxOperatorID, int, int, bexfun_t);
BALTAM_API int bxRegisterTernaryOperatorID(const char *, bxOperatorID, int, int, int, bexfun_t);

// BALTAM 插件函数原型
BALTAM_API int bxPluginInitLib(void*);
BALTAM_API int bxPluginInit(int, const bxArray *[]);
BALTAM_API int bxPluginFini();
BALTAM_API bexfun_info_t *bxPluginFunctions();

// 向内核和前端的轮询函数
BALTAM_API void * bxK2FQuery(const char *, void *);
BALTAM_API void * bxF2KQuery(const char *, void *);

#ifdef __cplusplus
}
#endif // of __cplusplus

#endif

