/*
 * =====================================================================================
 *
 *       Filename:  test_bex_api.cpp
 *
 *    Description:  source for test bex APIs
 *
 *        Version:  1.0
 *        Created:  12/17/2021 08:31:59 PM
 *       Revision:  none
 *       Compiler:  g++
 *
 *         Author:  Haoyang Liu (@liuhy), liuhaoyang@pku.edu.cn
 *   Organization:  BICMR
 *
 * =====================================================================================
 */

#include <iostream>
#include "ba_obj/ba_obj.h"
#include "print/obj2str.h"
#include "bex/bex.hpp"

using namespace baltam;

int main(){
    using internal::obj2str;
    bxArray *A = bxCreateDoubleMatrix(3, 3, bxREAL);
    double *pA = bxGetDoubles(A);

    for (baIndex i = 0; i < 9; ++i)
        pA[i] = i + 1;

    std::cout << "A =" << std::endl;
    std::cout << obj2str(*reinterpret_cast<ba_obj*>(A));

    bxResize(A, 2, 4);
    std::cout << "bxResize(A, 2, 4) =" << std::endl;
    std::cout << obj2str(*reinterpret_cast<ba_obj*>(A));

    const char *p_str[] = {"hello", "world", "baltam", "北太振寰"};

    bxArray *SS = bxCreateCharMatrixFromStrings(4, p_str);
    std::cout << "SS =" << std::endl;
    std::cout << obj2str(*reinterpret_cast<ba_obj*>(SS));

    bxDestroyArray(A);
    bxDestroyArray(SS);

    // BUG: conservative_resize() 对空矩阵
    bxArray *A1 = bxCreateDoubleMatrix(0, 0, bxREAL);
    bxResize(A1, 1, 1);
    std::cout << "bxResize(A1, 1, 1) =" << std::endl;
    std::cout << obj2str(*reinterpret_cast<ba_obj*>(A1));

    bxArray *A2 = bxCreateDoubleMatrix(0, 1, bxREAL);
    bxResize(A2, 1, 1);
    std::cout << "bxResize(A2, 1, 1) =" << std::endl;
    std::cout << obj2str(*reinterpret_cast<ba_obj*>(A2));
    bxResize(A2, 0, 1);
    std::cout << "bxResize(A2, 0, 1) =" << std::endl;
    std::cout << obj2str(*reinterpret_cast<ba_obj*>(A2));

    bxDestroyArray(A1);
    bxDestroyArray(A2);

    // ------------------------------------------------------------------
    // 测试：结构体
    // ------------------------------------------------------------------
    const char *fields[] = {"f", "g", "it"};
    bxArray *opts = bxCreateStructMatrix(1, 1, 3, fields);
    std::cout << "struct(\'f\', [], \'g\', [], \'it\', []) =" << std::endl;
    std::cout << obj2str(*reinterpret_cast<ba_obj*>(opts));

    bxArray *f = bxCreateDoubleScalar(1.1);
    bxArray *g = bxCreateDoubleScalar(0.8);
    bxArray *it = bxCreateInt32Scalar(100);

    bxSetField(opts, 0, "f", f);
    bxSetField(opts, 0, "g", g);
    bxSetField(opts, 0, "it", it);

    std::cout << "opts =" <<std::endl;
    std::cout << obj2str(*reinterpret_cast<ba_obj*>(opts));

    // 增加
    bxArray *msg = bxCreateString("hello world");
    bxSetField(opts, 0, "msg", msg);
    std::cout << "opts =" <<std::endl;
    std::cout << obj2str(*reinterpret_cast<ba_obj*>(opts));

    // 获取基本信息
    baSize n_opt = bxGetNumberOfFields(opts);
    std::vector<std::string> fieldnames = bxGetFieldNames(opts);
    std::cout << "FIELD 个数为：" << n_opt << std::endl;
    for(auto &i : fieldnames){
        std::cout << i << std::endl;
    }

    // 删除
    bxRemoveField(opts, "f");
    std::cout << "opts =" <<std::endl;
    std::cout << obj2str(*reinterpret_cast<ba_obj*>(opts));

    bxDestroyArray(opts);

    // ------------------------------------------------------------------
    // 测试：cell
    // ------------------------------------------------------------------
    bxArray *C = bxCreateCellMatrix(2, 3);
    bxArray *c1 = bxCreateDoubleScalar(1.0);
    bxArray *c2 = bxCreateInt64Scalar(65535);
    bxArray *c3 = bxCreateString("hello world");

    // 设置
    baIndex ind[] = {1, 1};
    bxSetCell(C, bxCalcSingleSubscript(C, 2, ind), c1);
    ind[0] = 2; ind[1] = 2;
    bxSetCell(C, bxCalcSingleSubscript(C, 2, ind), c2);
    ind[0] = 1; ind[1] = 3;
    bxSetCell(C, bxCalcSingleSubscript(C, 2, ind), c3);
    std::cout << "C =" << std::endl;
    std::cout << obj2str(*reinterpret_cast<ba_obj*>(C));

    bxDestroyArray(C);

    // ------------------------------------------------------------------
    // 测试：string array
    // ------------------------------------------------------------------
    const char *pstr[] = {"one", "two", "three", "four"};
    bxArray *S = bxCreateStringMatrixFromStrings(2, 2, pstr);
    std::cout << obj2str(*reinterpret_cast<ba_obj*>(S));

    // 设置
    bxSetString(S, 0, "ONE, 1");
    bxSetString(S, 2, "THREE, 3");
    std::cout << obj2str(*reinterpret_cast<ba_obj*>(S));

    bxDestroyArray(S);
    
    return 0;
}
