function [ w ] = Alg_11_3_Linear_Finite_Difference( p, q, r, a, b, alpha, beta, N )
% numerical analysis(9th edition): algorithm 11.3
% input:
% functions p, q, r s.t. y'' = p*y'+q*y+r
%   endpoints a, b
%   boundary conditions alpha, beta
%   integer N
% output:
%   approximations w = [w0 w1 ... wN] to y = [y(x0) y(x1) ... y(xN)]
%       with xi = a+ih, i = 0,...,N, h = (b-a)/N
% !
%   h = (b-a)/N, differs from the algorithm on the book,
%       which gives h = (b-a)/(N+1)

    h = (b-a)/N;
    % construct tridiagonal matrix and rhs
    d = ones(1,N+1); u = zeros(1,N); l = zeros(1,N);
    rhs = zeros(1,N+1); rhs(1) = alpha; rhs(N+1) = beta;
    for i = 2:N
        x = a+(i-1)*h;
        d(i) = 2+h^2*q(x);
        l(i-1) = -0.5*h*p(x)-1; u(i) = 0.5*h*p(x)-1;
        rhs(i) = -h^2*r(x);
    end
    % solve the matrix system
    lii = d(1); u(1) = u(1)/lii; rhs(1) = rhs(1)/lii;
    for i = 2:N+1
        lii = d(i)-l(i-1)*u(i-1);
        if i<N+1
            u(i) = u(i)/lii;
        end
        rhs(i) = (rhs(i)-rhs(i-1)*l(i-1))/lii;
    end
    for i = N:-1:1
        rhs(i) = rhs(i)-u(i)*rhs(i+1);
    end
    w = rhs;
end
