function [ t, w, h, flag ] = Alg_5_3_Runge_Kutta_Fehlberg( f, a, b, alpha, TOL, hmax, hmin )
% numerical analysis(9th edition): algorithm 5.3
% input:
%   function y' = f
%   endpoints x: [a, b]
%   initial condition alpha
%   tolerance TOL
%   maximum step size hmax
%   minimum step size hmin
% output:
%   t, w, h where w approximates y(t) and the step size h was used, 
%       or a message that the minimum step size was exceeded
%   flag: 1-the minimum step size was exceeded
% !
%   differ from the algorithm from book
%       wheater repeat calculation depends on q but not |w1-w1|/h

    t = zeros(1,10000); w = zeros(1,10000); h = zeros(1,10000);
    flag = 0;
    t(1) = a; w(1) = alpha; h(1) = hmax;
    p = 1;
    tmp = 1;
    while t(p)<b && tmp<=10000
        h_now = h(p);
        k1 = h_now*f(t(p),w(p));
        k2 = h_now*f(t(p)+h_now/4,w(p)+k1/4);
        k3 = h_now*f(t(p)+h_now*3/8,w(p)+k1*3/32+k2*9/32);
        k4 = h_now*f(t(p)+h_now*12/13,w(p)+k1*1932/2197-k2*7200/2197+k3*7296/2197);
        k5 = h_now*f(t(p)+h_now,w(p)+k1*439/216-k2*8+k3*3680/513-k4*845/4104);
        k6 = h_now*f(t(p)+h_now/2,w(p)-k1*8/27+k2*2-k3*3544/2565+k4*1859/4104-k5*11/40);
        w1 = w(p)+k1*25/216+k3*1408/2565+k4*2197/4104-k5/5;
        w2 = w(p)+k1*16/135+k3*6656/12825+k4*28561/56430-k5*9/50+k6*2/55;
        q = .84*sqrt(sqrt(TOL*h_now/abs(w1-w2)));
        if q>=1
            p = p+1;
            t(p) = t(p-1)+h_now;
            w(p) = w1;
            h(p) = h_now*q;
            if t(p)+h(p)>b
                h(p) = b-t(p);
            end
        else
            h(p) = h_now*q;
            if h(p)<hmin
                fprintf('the minimum step size was exceeded\n');
                flag = 1;
                break;
            end
        end
        tmp = tmp+1;
    end
    t = t(1:length(find(h))+1);
    w = w(1:length(t));
    h = h(1:length(t)-1);
end
