function [ L, x ] = Alg_6_6_Cholesky( n, A, b )
% numerical analysis(9th edition): algorithm 6.6
% input:
%   dimension n
%   the entries aij, 1<=i,j<=n of A
%   b: Ax = b 
% output:
%   the entries lij, 1<=j<=i, 1<=i<=n of L
%   x: the solution of Ax = b

    L = zeros(n);
    for i = 1:n
        if i==1
            L(i,i) = sqrt(A(i,i));
        else
            L(i,i) = sqrt(A(i,i)-L(i,1:i-1)*L(i,1:i-1)');
        end
        for j = i+1:n
            if i==1
                L(j,i) = A(j,i)/L(i,i);
            else
                L(j,i) = (A(j,i)-L(j,1:i-1)*L(i,1:i-1)')/L(i,i);
            end
        end
    end
    % modified version which solves system Ax = b
    x = zeros(n,1);
    for i = 1:n
        x(i) = (b(i)-L(i,1:i-1)*x(1:i-1))/L(i,i);
    end
    for i = n:-1:1
        x(i) = (x(i)-L(i+1:n,i)'*x(i+1:n))/L(i,i);
    end
end