%大地坐标系(BLH) --> 地固坐标系(x,y,z) 
% 例:
% BLH = [39, 116, 235]  
% %这里的235可以理解为距离地球表面的高度
% BLH2xyz(BLH) 
% % 得到的结果是
% 1e6 * 
% [-2.175859556748517   4.461173207938506 3.992464912897231]
% 可以简单判断一下，90°E~179°E区域x为负，北半球区域y为正
function xyz = BLH2xyz( BLH, rad )

	if(nargin < 2) 
		fprintf("输入BL是角度，而不是弧度");
		rad = false
	end	

	if( nargin < 1 )
		error("至少输入一个参数\n");	
	end	

	if(size(BLH,2) ~= 3) 
		fprintf("现在输入BLH的size是%4.0fx%4.0f\n", size(BLH,1),size(BLH,2))
		error("输入一个n x 3 的矩阵, n > = 1\n");
	end
	B = BLH(:,1); 
	L = BLH(:,2); 
	H = BLH(:,3); 
	
	if ~rad
		B =  degree2rad(B);
		L =  degree2rad(L);
	end

	a = 6378137.0000 ; % a 为地球椭球的长半轴, 单位是米
	b = 6356752.3141; % b 为地球椭球的短半轴, 单位是米
	e2 = 1 - (b/a)^2; 	
	N =  a / sqrt( 1 - e2 * (sin(B)).^2); 

	x = (N+H).*cos(B).*cos(L);
	y = (N+H).*cos(B).*sin(L);
	z = ((1-e2)*N + H) .* sin(B);
	xyz = [ x,y, z] ;
end


function r = degree2rad(degree)
	r = degree/180 *pi ;
end

