% 为圆形生成 easymesh 的输入文件
% 输入参数filename: 需要是字符矩阵类型，指定输出的文件名, 例如 'test_output', 会得到 test_output.d 
% 输入参数N: 圆周均匀剖分的份数
% 输入参数M: 在state=2的时候采用到，圆周的弦长的 1/M 作为剖分尺寸
% 输入参数state: state = 1  时，M 没有用到，用的是圆周均分后的弦长d作为网格尺寸, state =2 时,
%                   用的是 d/M
% 输入参数R: 圆的半径
% 输出: 是一个文件名是 filename.d 的txt文件
% 例:  GMS('test1', 10, 3, 2, 1) 
%  %把单位圆的周长剖分10份，state=2意味网格尺寸是 弦长的1/3
%  GMS('test2', 10, 1, 1, 1) 
%  %把单位圆的周长剖分10份，state=1意味网格尺寸 就是弦长, 此时的第三个参数的值没有被用到
function GMS(filename,N,M,state,R)
	fid = fopen([filename '.d'],'w');
	fprintf(fid,'#--------------------------------------------#\n');
	fprintf(fid,'# circle.d: circle with %d segment elges #\n',N);
	fprintf(fid,'#--------------------------------------------#\n');
	fprintf(fid,'\n');
	fprintf(fid,'#=============\n');
	fprintf(fid,'| POINTS     |\n');
	fprintf(fid,'=============#\n');
	fprintf(fid,'%d # number of points #\n\n',N);
	fprintf(fid,'# Nodes which define the boundary #\n');
	steplength = 2*pi/N;
	X=R * cos(0:steplength:2*pi);
	Y=R * sin(0:steplength:2*pi);
	for n = 0:N-1,
		switch (state)
			case 1,
				l = sqrt((X(n+2)-X(n+1))^2+(Y(n+2)-Y(n+1))^2);
				l = l/M;
				fprintf(fid,'%d:   %f   %f   %f   %d\n',n,X(n+1),Y(n+1),l,n+1);
			case 2,
				fprintf(fid,'%d:   %f   %f   %d   %d\n',n,X(n+1),Y(n+1),M,n+1);
			otherwise,
				disp('error(GMS): argment 3 is not 1 or 2');
				disp('	1 for easymesh, 2 for genmesh');
				return;
		end
	end
	fprintf(fid,'\n#============\n');
	fprintf(fid,'| SEGMENTS   |\n');
	fprintf(fid,'=============#\n');
	fprintf(fid,'%d # number of segments #\n\n',N);
	fprintf(fid,'# Boundary segments #\n');
	for n = 0:N-2,
		fprintf(fid,'%d:   %d   %d   %d\n',n,n,n+1,n+1);
	end
	fprintf(fid,'%d:   %d   0   %d\n',N-1,N-1,N);
	fclose(fid)
