%北太天元的代码 解析法求解卫星定位问题
% (x-x_i)^2 + (y-y_i)^2 + (z-z_i)^2 - c^2(d-t_i)^2  = 0 , i = 1,2,3,4
% 其中 (x_i,y_i,z_i) 是第i个卫星位置，t_i是第i个卫星信号传输时间（带有偏差) 是已知的
% (x,y,z) 是接收器的位置，d 是传输时间偏差, (x,y,z,d)是要求的未知数
% 这里先把 x,y,z 都表示成d的表达式，然后代入某一个方程，得到的d的一元二次方程，
% 通过一元二次方程求根公式得到d，然后求出 x,y,z 

%四个卫星的坐标
%卫星位置 
A = [
 15600,7540,20140;   %第一个卫星的 x, y, z 坐标
 18760,2750,18610 ;
 17610,14630,13480 ;
 19170,610,18390 ];
%四个卫星的信号的传输时间（带有偏差的传输时间)
t = [0.07074,0.07220,0.07690,0.07242];
c = 299792.458;% 光速，单位 千米/秒

At = [A, t'] * diag([1,1,1,c]);

tsq = (t.^2)';

Atsql = At.^2 * diag([ 1, 1, 1, -1 ]); 
W = sum( Atsql(1,:) - Atsql(2:4,:) , 2 );  % sum( A, 2) 会对A矩阵的每行求和，得到一个列向量

UUU = (At(2:4,:)-At(1,:)) * diag( [ 2, 2, 2, -2*c] );

UW = [UUU,W];

[A1,B1,C1] = deal(A(1,1), A(1,2), A(1,3)); 

L1 = det(UW(:,[2,3,1]));
L2 = det(UW(:,[2,3,4]));
L3 = det(UW(:,[2,3,5]));

M1 = det(UW(:,[1,3,2]));
M2 = det(UW(:,[1,3,4]));
M3 = det(UW(:,[1,3,5]));

N1 = det(UW(:,[1,2,3]));
N2 = det(UW(:,[1,2,4]));
N3 = det(UW(:,[1,2,5]));

E1=-L2/L1;
E2=-L3/L1;
E3=-M2/M1;
E4=-M3/M1;
E5=-N2/N1;
E6=-N3/N1;

G=E1^2+E3^2+E5^2-c^2;
H=2*(E1*E2+E3*E4+E5*E6-A1*E1-B1*E3-C1*E5+c^2*t(1));
I=E2^2+E4^2+E6^2-2*(A1*E2+B1*E4+C1*E6)-c^2*tsq(1)+A1^2+B1^2+C1^2;


d1=(-H+(H^2-4*G*I)^.5)/(2*G)
d2=(-H-(H^2-4*G*I)^.5)/(2*G)

x1=E1*d1+E2;
x2=E1*d2+E2;
y1=E3*d1+E4;
y2=E3*d2+E4;
z1=E5*d1+E6;
z2=E5*d2+E6;

%计算出来的两个位置, 我们一般选择距离地球表面近的位置
[ [x1;y1;z1;] , [x2;y2;z2] ]
