% 牛顿法求解多元非线性方程组
% 非线性方程组来源于四星定位问题
%  F(X) = 0; 其中 X = (x,y,z,d)' ,
% F(X)  = (F_1(X), ..., F_4(X))'
% F_i(X) = (x- x_i )^2 + (y-y_i)^2 + (z-z_i)^2 - ( c (t_i-d)^2 ) 
% 其中 (x_i,y_i,z_i) 是 第 i 个卫星的位置, t_i 是信号传输的时间 (但是这个时间不准确，
% 具体的偏差是d (是未知的)， 因此 t_i - d 是 第i个卫星信号传输的真实时间 

%Guess 是猜测的初值 
%卫星位置  是一个 nx3的矩阵，n 是卫星个数， 第i行的3列对应第i个卫星的(x,y,z)坐标
%t 是卫星传输的测量数据, 是一个长度是 n 的向量， n 是卫星的个数

function [Xk, err, Iterations] = GNNS_Newton_method_Ex1(Guess,卫星位置,t)

	Xk=(Guess(:))'; % 确保Xk 是一个行向量 (x,y,z,t)

	if size(卫星位置,1) ~= numel(t) 
		fprintf("卫星位置计算的卫星个数 %6.0f == 传输时间t 的数据个数 %6.0f 不成立", ...
		 size(卫星位置,1), numel(t) ); 	
		return;
	end
	卫星位置和时间 = [ 卫星位置 ,  t(:)]   

	%c 是光速 , 单位 千米/秒
	global c;
	c=299792.458;


	%F 卫星定位问题的向量值函数
	%D F(x,y,z,d) 的jacobian矩阵

	F=获得函数向量F(Xk,卫星位置和时间);
	J=获得Jacobian矩阵J(Xk,卫星位置和时间);

	%迭代可以通过误差忍量和最大迭代步数来控制
	% 
	Max_it = 30;  %最大迭代步数
	torrence = 1e-6; % 忍量
	Iterations=1;

	while Iterations < Max_it 
    %V  -(Jacobian Matrix) \ F(目前近似解Xk) 
    V=(J\(-F));
    % 下一步的近似解是在的目前的近似解上增加V
    Xk = Xk+V'; % V 是一个列向量, 因此 V先转置 再加在 Xk 上

    
    F=获得函数向量F(Xk,卫星位置和时间);
    D=获得Jacobian矩阵J(Xk,卫星位置和时间);
    
		% 误差 = F 在新的近似解上的值的 无穷范数
    err= norm(F, 'inf');
		if err < 1e-6
		 	break; 
		end
    %迭代步数增加1
    Iterations=Iterations+1;
	end
%三个返回值
%err;
%Iterations;
%Xk;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 计算卫星定位的多元向量值函数, 返回的向量 F = m x1 ， 
% 第i行是 (x-x_i)^2 + (y_y_i)^2 + (z-z_i)^2 - c^2 (d - t_i)^2 
% 其中 (x,y,z) 是接收器的位置(未知)，
% (x_i,y_i,z_i) 是第i个卫星的位置(已知)
% d 是接收器的时间偏差(又成为钟差), 
% t_i 是信号 从i个卫星传输到 接收器 所用时间的读数 (不是真实时间，是有偏差的)
% Xk 是当前迭代步的接收器的时间和位置 (x,y,z,d);
%卫星位置  是一个 nx3的矩阵，n 是卫星个数， 第i行的3列对应第i个卫星的(x,y,z)坐标
function F = 获得函数向量F(Xk,卫星位置和时间)
	global c; %光速
	F = Xk- 卫星位置和时间;  % Xk 是一个行向量 [ x,y,z,t ] ; 
	% 卫星位置和时间 是一个矩阵 mx4  , Xk- 卫星位置和时间 会得到 mx4 的矩阵 ，
	% 相当于把Xk 行copy成了 mx4的矩阵,然后再去减 卫星位置和时间
	w = [ 1; 1;  1;  -c*c] ;
	F = F.^2 * w;   % 第i行是 (x-x_i)^2 + (y_y_i)^2 + (z-z_i)^2 - c^2 (d -t)^2 . 
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 计算 卫星定位的多元向量值函数的jocobian矩阵
%卫星位置和时间 是一个 mx4的矩阵， 每一行存放一个卫星的(x,y,z)位置和传输时间 t
%Xk  第k个迭代步的 (x,y,z,d);
%卫星个数 is the amount of satellites (equal to the length of the vector A)

function J=获得Jacobian矩阵J(Xk,卫星位置和时间)
	global c;  %光速
	D = Xk - 卫星位置和时间;
	w = diag([2,2,2,-2*c^2]);
	J = D*w; 
end


