%朴素贝叶斯算法挑瓜的例子
%假设我们通过购买了一批瓜，试吃后得到的数据如下
/*
颜色(1 青绿 2 黑色 3 黄),回响 (1 清脆 2 浑浊),触感 (1 光滑， 2 粗糙_,形状 (1 圆形， 2 扁),"质量 (1 好瓜, 2 坏瓜)"
1,1,1,1,1
1,2,2,2,2
2,1,1,1,1
1,1,2,2,2
3,1,1,1,2
3,1,2,1,2
1,1,1,1,1
3,1,1,2,2
2,2,1,1,1
2,1,2,2,1
2,2,1,1,1
2,1,1,1,1
*/
%上面的数据我们保存在bayesEx2Data.xlsx中

clear; clc; close all; 

%读入数据，最后一列为瓜的质量
data = readmatrix("bayesEx2Data.xlsx", 'Range', 'A2:E13');

[n,m] = size(data);
label_data = data(:,m);

prompt_msg = ['请输入质量未知的瓜的属性, ', ...
  '\n 例如青绿、清脆、粗糙、圆形的瓜', ...
  '就输入向量[1,1,2,1]:  \n'];
X=input(prompt_msg);

if( length(X) ~= m-1) 
  error("属性的个数不足");
end

for i=1:m-1
	if ~ismember(X(i),data(:,i))
		error(sprintf("输入的第%d的属性值不符合要求",i));  
	end
end

[p_xianyan, label_unique] = prior_p(label_data)
n_label = length(label_unique) ; %标签的种数

%计算条件概率概率 P(X|label_i)
for i=1:n_label 
  %目前处理的是第i种标签label_i
	label_i_row = (label_data == label_unique(i));   
	data_i = data(label_i_row,:); % 第i种标签所在的行构成新数据子集
  n_data_i = length(data_i);
	pj = zeros(m-1,1);
	for j=1:m-1 
		pj(j) = sum( data_i(:,j) == X(j))/n_data_i; % 计算p(Xj|label_i)
	end	
	p(i)=prior_p(i)*prod(pj); % 计算p(label_i)*p(X|label_i)
end

[mx,mxpos]=max(p);
kind = label_unique(mxpos);
mx = mx/sum(p);
sprintf("输入的瓜质量属于第 %d 等级的概率最大，其概率为 %f \n", kind, mx)

%计算先验概率
% data_label 是数据的标签列构成的向量
% label_unique 返回标签可能取值个数，如这个例子只有好瓜和坏瓜
% 两种，那么label_uique = [1,2] 
% p_xianyan 返回每种瓜的先验概率
function [p_xianyan, label_unique]  = prior_p(data_label)
  n_sample = length(data_label); %样本的数量
  label_unique = unique(data_label); %查看瓜的质量分几个等级
  n_label = length(label_unique); % 这个数据等于瓜的等级个数
  p_xianyan = zeros(1, n_label); 
  for i=1:n_label
    p_xianyan(i) = sum(data_label == label_unique(i))/n_sample;   
  end
end  

