clc; clf; close all; clear all; 
% easymesh 是一个free的网格剖分程序，可以把它加入北大天元
% 做成插件, 然后从北大天元调用。
load_plugin("easymesh")
% 四个点的坐标分别是 (0,0), (1,0), (1,1), (0,1)
points_part1 = [ 0 0 ;  1 0;  1 1 ;  0 1 ] ;
points_part2 =  ones(size(points_part1))*0.3 + 0.2*points_part1; 
points = [ points_part1 ; points_part2];
%四个边, 
% edges 是 4x2的矩阵，
% 第一行: 0 1 表示从第一个点(0,0)指向第二个点(1,0)的一条边
% 第二行: 1 2 表示从第二个点(1,0)指向第三个点(1,1)的一条边
edges_part1 = [ 0  1;  1  2; 2 3 ; 3 0 ] ;  %逆时针的闭合曲线
edges_part2 = 4 + [ 0  1;  1  2; 2 3 ; 3 0 ] ;  
%中间形成一个洞，边需要按照顺时针顺序排列
edges_part2 = edges_part2(end:-1:1, :) ; % 这一步也不能少 
edges_part2 = edges_part2(:, end:-1:1) ; % edges_parts2 是顺时针的闭合曲线
%两部分边和在一起
edges = [edges_part1; edges_part2] ;
[np,~] = size(points);  %点的个数
[ns,~] = size(edges);  %边的个数
%每个点附近网格大小的参数，这里都设置为 0.1
nodes_F = 0.1*ones(np,1);
%点的标志，前4个点设置 1, 后四个点设置为 2
nodes_mark(1:4) = 1; 
nodes_mark(5:8) = 2;
%边的标志，前四个边设置 1, 后四个边设置为 2
edges_mark(1:4) = 1;
edges_mark(5:8) = 2;

hold on
for i=1:4
	line( ... 
		[ points(edges(i,1)+1,1), points(edges(i,2)+1,1)] , ...
		[ points(edges(i,1)+1,2), points(edges(i,2)+1,2)]) 
end 
input("Press any key to continue");
for i=5:8
	line( ... 
		[ points(edges(i,1)+1,1), points(edges(i,2)+1,1)] , ...
		[ points(edges(i,1)+1,2), points(edges(i,2)+1,2)]) 
end 
input("Press any key to continue");
hold off
clf; 
close all

%剖分网格，生成的网格保存在第一个参数知名的三个文件中
%分别是 example5.n , example5.s, example5.e 分别存储的是点，边和单元的信息
[mesh_nodes,mesh_elements,mesh_sides,mesh_nodes_mark,mesh_sides_mark]  = easymesh2d("example5", points, edges, nodes_F, nodes_mark, edges_mark);

%返回的单元 mesh_elements 是 nele x 3 的矩阵，每行的3个元素是单元的节点编号(从0开始编号)
%返回的单元 side_elements 是 nside x 2 的矩阵，每行的2个元素是边的节点编号(从0开始编号)
[nnode, ~] = size(mesh_nodes);
[nside, ~] = size(mesh_sides); 
hold on
for i=1:nside
	ii = mesh_sides(i,1)+1; % 边起点的编号（从1开始计数）
	jj = mesh_sides(i,2)+1; % 边终点的编号（从1开始计数）
	if( ii ~= jj && ii <= nnode && jj <= nnode)
		line( ... 
			[ mesh_nodes(ii,1), mesh_nodes(jj,1)] , ...
			[ mesh_nodes(ii,2), mesh_nodes(jj,2)] ) ;
	else
		sprintf("side %d: %f %f", i, mesh_sides(i,1)+1, mesh_sides(i,2)+1)  
	end
end 
hold off

