% fermi_dirac 积分
% 可以参考视频
% 【北太天元上的Fermi-Dirac积分】 https://www.bilibili.com/video/BV1wG411F7kT/?share_source=copy_web&vd_source=2adc5aa7a702b808eb8b31dbd210f954

%视频里出现了 
%花体的\mathcal{F}_{1/2}(x)
% = 1/\gamma(1/2+1) \int_{0}^{\infty} \frac{ t^{1/2} }{1+exp(t-x)} dt
%正体的F_{1/2}(x)
% = \int_{0}^{\infty} \frac{ t^{1/2} }{1+exp(t-x)} dt
%二者相差一个因子  1/\sqrt{3/2} = 2/\sqrt{\pi} \approx 1.1284
%北太天元里的 fermi_dirac_half 相当于花体的\mathcal{F}_{1/2}
% 这里提供 \mathcal{F}_{1/2} 函数的反函数的一个近似表达式

fh= @(x) log(x)+0.353553*x-4.95009e-3* x .* x ... 
  +1.48386e-4 .*x .*x .*x-4.42563e-6*x .*x .*x .*x;

x = -2:.1:15;
y_onehalf = fermi_dirac_half( int32(1), x);

x_fh =  fh(y_onehalf); %近似公式求解fermi_dirac_half 的反函数 

figure(1)
plot(y_onehalf,x,'k-',y_onehalf,x_fh,'r*')
xlabel("fermi_dirac_half(1,x)")
ylabel("x");
legend('fermi_dirac_half的反函数', 'fermi_dirac_half的近似反函数')
title("从纵轴的标的数字来看，二者在eta_F <6 时近似公式还算准确")

plot(y_onehalf, x-log(y_onehalf),'k-', y_onehalf, x_fh-log(y_onehalf),'r*' )

figure(2)
% Sommerfeld approximation 近似公式在 电子数密度高的时候 有效
gh = @(x)  (9*pi/16)^(1/3)*x.^(2/3),  
x_gh = gh(y_onehalf);

len = numel(y_onehalf);
ind = 1:5:len;
plot(y_onehalf(ind), x(ind),'k-', 'LineWidth', 5, ...
	y_onehalf(ind), x_fh(ind),'b*', ... 
	y_onehalf(ind), x_gh(ind),'r*' )
legend('fermi-dirac of order 1/2的反函数', ... 
	'Joyce-Dixon 近似公式求fermi-dirac of order 1/2 的反函数', ...
	'Sommerfeld approximation 求fermi-dirac of order 1/2 的反函数')
title("Fermi-Dirac of order 1/2的反函数")

figure(3)
plot( y_onehalf(ind), log10( abs((x_fh(ind)-x(ind))./x(ind) )),'b*', ... 
	y_onehalf(ind), log10(abs( (x_gh(ind)-x(ind))./x(ind) )),'r*' )
legend( 'Joyce-Dixon 近似公式求fermi-dirac of order 1/2 的反函数 的相对误差', ...
	'Sommerfeld approximation 求fermi-dirac of order 1/2 的反函数 的相对误差')
title("Fermi-Dirac of order 1/2的反函数 的误差")





