%绘制男女生身高的Gaussian Mixture Model (GMM)
clc; clf; close all; clear all;
%男女生共取1000人，女生平均身高161，男声平均身高175
male=175+sqrt(10)*randn(1,500); %产生均值为175，方差为10的一个1*100的随机数
female=161+sqrt(10)*randn(1,500); %产生均值为161，方差为10的一个1*100的随机数
height=[female male];

%画出生成的男女性的频率分布直方图
figure(1)
hist(female,10);
hold on
hist(male,10);%画出生成的男性的频率分布直方图
title('某校男女生身高的分布直方图');
xlabel('身高(cm)');
ylabel('人数(个)');
hold off;

%GMM的构造
%首先根据经验分别对男女生身高的均值、方差和权值进行初始化
t1_old.mu = 170; t1_old.sigma = 10; t1_old.w = 0.8; %男生
t2_old.mu = 160; t2_old.sigma = 10; t2_old.w = 0.2; %女生

n_iter = 40; %设置迭代次数
male_w = zeros(n_iter,1); %定义一个数组来存储每次的迭代结果
male_w(1) = t1_old.w;
%将第一列存储初始值

%开始迭代
for i=1:n_iter-1
    [t1_new,t2_new] = emEx1(height,t1_old,t2_old);
    t1_old = t1_new; t2_old = t2_new; 
		%可以加入其他的判断来提前结束迭代
    male_w(i+1) = t1_new.w; %记录第i+1次迭代得到的男生权重
end

%画出男女生的权重迭代历史
figure(2);
plot(1:n_iter, male_w, 'b-+'); %画出男生的权重迭代历史
h1 = gco;   %取出所画线段的句柄
set(h1,'LineWidth', 3); %然后设置该线段的宽度为3
hold on;
plot(1:n_iter, 1-male_w,'r-*','LineWidth', 3); %画出女生权重迭代历史
legend('男生权重变化','女生权重变化','Location','NorthEast'); %坐标轴设置，将图例放置在图的右上角
title('男女生的权重随着迭代次数的变化');
xlabel('迭代次数');
ylabel('权重');
axis([1 n_iter 0 1]);
hold off

figure(3);
hold on;
%男生女生以及混合后身高的概率密度曲线 
x = linspace(130,220,181);% 在1.35m和2.20m之间等距取181个点
y1 = normpdfEx1(x, t1_new.mu, t1_new.sigma); %男生的概率密度函数
y2 = normpdfEx1(x, t2_new.mu, t2_new.sigma); %女生的概率密度函数
y3 = t1_new.w*y1+t2_new.w*y2; %男生女生混合的概率密度
plot(x, y1, 'r', 'LineWidth', 1.5); %LineWidth 目前的L和W都需要大写
plot(x, y2, 'g', 'LineWidth', 1.5); 
plot(x, y3, 'b', 'LineWidth', 1.5); 
legend('男生','女生','混合');
title('男生女生以及混合后身高的概率密度曲线');
xlabel('身高(cm)'); ylabel('概率');
hold off;

