%卡曼滤波的例子 
% 考虑线性动力系统
% x(t+1) = Ax(t) + Gw(t)  
% 其中 x(1), x(2), ..., 是随机变量
% w(t) 是独立同分布的随机变量，服从正态分布
% A 和G是给定的矩阵
% 观察方程
% z(t) = Hx(t) + Rv(t)
% 其中H和G是给定的矩阵, 
% v(t)是独立同分布的随机变量, 服从正态分布
%
N=120; %计算的时间范围 t=1,...,N
Xexact =25*ones(1,N); % N个时刻的温度（假设温度是不变的）
X=zeros(1,N);%存由动力系统计算的温度(称为预测温度)  
X_估计=zeros(1,N); %存放卡曼滤波修正后的温度
Z=zeros(1,N); %存放观测的温度
P=zeros(1,N); %存温度的方差
X(1)=25.1; %给定温度的初值
P(1)=0.01; %温度方差的初值
Z(1)=24.9; %观测值的初值
X_估计(1)=Z(1);
I=eye(1);

%动力系统的参数 A, G
ds.A = 1;
ds.G = 1;
ds.Q = 0.01;
ds.W=sqrt(ds.Q)*randn(1,N);
%观察的参数 H
obs.H = 1; 
obs.R = 0.25;
obs.V=sqrt(obs.R)*randn(1,N);

% Kalman Filter
for k=2:N
    X(k)=ds.A*X(k-1)+ds.G*ds.W(k-1); %房间温度的真实值
    %在真实情况下，房间的真实温度值应该是不知道的，
    %因此下面的Z(k) 不使用 X(k)计算，而应该直接读取观测值
    Z(k)=obs.H*X(k)+obs.V(k); %观测值 
    X_预测=ds.A*X_估计(k-1);           
    P_预测=ds.A*P(k-1)*(ds.A)'+ds.Q;        
    Kg=P_预测*(obs.H)'*inv(obs.H*P_预测*(obs.H)'+obs.R); 
    e=Z(k)-obs.H*X_预测;            
    X_估计(k)=X_预测+Kg*e;         
    P(k)=(I-Kg*obs.H)*P_预测;
end
 
%误差分析 
Err_Messure=abs(Z-X);
Err_Kalman=abs(X_估计-X);

t=1:N;
figure(1)
plot(t, Xexact,'-b', 'LineWidth', 3 , ... 
  t, X, '-r', 'LineWidth', 2 , ... 
  t, Z, '-k', 'LineWidth', 2 ,  ...
  t, X_估计, '-g', 'LineWidth', 2.5);
legend('精确值','动力系统计算值','测量值','卡曼滤波估计值');         
xlabel('时间');
ylabel('温度');
title('卡曼滤波');

figure(2)
plot(t, Err_Messure, '-b', 'LineWidth', 2, ...
  t, Err_Kalman, '-k', 'LineWidth', 2.5);
legend('测量误差','卡曼滤波误差');         
xlabel('时间');
ylabel('误差');
title('误差分析');
