% k-紧邻方法预测验证数据的标签 
% 训练数据文件名: 这是使用别的方法已经训练出来的数据，最后一列是标签
%                其他各列是数据的特征(feature)
% 验证数据文件名: 最后一个列是标签，用来验证预测的标签是否正确
% k:  k-紧邻方法选择的邻居个数
% 用法举例: knnEx1main"pendigits_train.csv", "pendigits_test.csv", 2);
function knnEx1main(训练数据文件名, 验证数据文件名, k)
		if nargin == 0 
			训练数据文件名 = "pendigits_train.csv";
			验证数据文件名 = "pendigits_test.csv";
			k = 1;
		elseif nargin == 2 
			k = 1;
		else
			error("knnEx1main 输入参数不正确, 请输入3个输入参数");
		end

    %读入数据
    训练数据 = readmatrix(训练数据文件名,"NumHeaderLines",1);
    验证数据 = readmatrix(验证数据文件名, "NumHeaderLines",1);
    训练数据标签 = 训练数据(:, end);
    验证数据标签 = 验证数据(:, end);
    训练数据特征 = 训练数据(:, 1: end-1);
    验证数据特征 = 验证数据(:, 1: end-1);

		%训练数据和验证数据的特征(features)做标准化
    均值 = mean(训练数据特征);
    标准差 = std(训练数据特征, 1);
    训练数据特征 = 标准化(训练数据特征, 均值, 标准差); 
    验证数据特征 = 标准化(验证数据特征, 均值, 标准差);

		%根据验证数据特征，利用k近邻算法预测验证数据的标签，然后和真实的验证数据对比
    准确率 =  knnEx1(训练数据特征, 验证数据特征, 训练数据标签, 验证数据标签, k);
		fprintf('分类准确率 = %6.4f \n', 准确率);
end

%data是一个多列的数据，每一列都做标准化
%mean_value 是每一列的均值放在一起构成的向量
%std_value 是每一列的标准差放在一起构成的向量
function [data] = 标准化(data, mean_value, std_value,)
    for i = 1: (size(data, 2))
        data(:, i) = data(:, i) - mean_value(1, i);
        data(:, i) = data(:, i) / std_value(1, i);
    end
end
