% logistic regression 逻辑回归的例子
% 和lrEx1.m 相比，我把最大似然函数算得简单了一点
% 得到 t 和 lrEx1i.m 中的t 很大不同，但是决策直线还是一致的，
% 这里计算的更加准确，因为最后计算的函数值更小  L  = - log ( 最大似然函数 ) 
% 数据有三列
% 语文、数学、是否被录取
% 例如 lrEx1data.txt 的数据如下
% 最后得到的是一个匿名函数表达的决策函数
/*
100 100 1
90 90 1
80 80 1
98 75 1
25 38 0 
28 32 0 
27 33 0 
22 33 0 
*/

[pre, t] = logistic_regression('lrEx1data.txt');
x1=30:100;
plot(x1,(-t(1)-t(2)*x1)/t(3),'LineWidth',2); %决策边界
legend('决策边界')
%画出决策边界:
% x1 语文, x2 数学, t(1)+x1*t(2)+x2*t(2) = 0 是决策边界 
% 也就是直线 x2 = (-t(1)x1 - t(2) )/t(3)
hold on
data=readmatrix('lrEx1Data.txt');
T = data( data(:,3) == 1, 1:2); %被录取的成绩
F = data( data(:,3) == 0, 1:2); %未被录取的成绩 
scatter(T(:,1),T(:,2),100,'b');
scatter(F(:,1),F(:,2),100,'r');
title('蓝点是被录取的成绩，红点是未被录取的成绩')
xlabel('语文成绩')
ylabel('数学成绩')
hold off

function [pre, t] = logistic_regression(datafile)
data=readmatrix('lrEx1Data.txt');
%用fminsearch函数最优化
%h(x) = logisticc(x) =@(t)((1.+exp(-(t(1)+dot(t(2:3),x)))).^-1)

/* 多行注释开始
*******************
下面是使用极大似然估计来求最好的参数
标签为 y_i = 1, 那么发生这个事件的概率是 h(x_i) (这是把上面的logisitic函数看成了概率) 
标签为 y_i = 0, 那么发生这个事件的概率是 1 - h(x_i) 
所有的观测的数据认为是独立事件，因此观察所有数据的概率是
p =  \prod_{ 对于 y_i =1 情况} h(x_i)  \prod_{对于 y_j=0 情况} (1-h(x_j)) 
下面是等价求了这个函数的对数，然后再除以n, 再乘上-1， 作为目标函数
我们只要求这个新的目标函数的最小值点即可。 
******************
对行注释结束 */

index1 = find(data(:,3) == 1) ; % 返回标签是1的数据index
index0 = find(data(:,3) == 0) ; % 返回标签是0的数据index

n=length(data(:,1)); %学生数
s=@(t) -(  ... 
     sum( log( (1.+exp(-(t(1) ...
		                     +data(index1,1)*t(2) ...
		                     +data(index1,2)*t(3) ...
										    ) ...
										  ) ...
							  ).^-1 ...
							) ... 
				)  +  ...
     sum( log( 1 - (1.+exp(-(t(1) ...
		                     +data(index0,1)*t(2) ...
		                     +data(index0,2)*t(3) ...
										    ) ...
										  ) ...
							  ).^-1 ...
							) ... 
				)   ...
		 )/n;
[t,fval] = fminsearch(s,[0,0,0]) %以初值t=[0,0,0]求函数s的最小值点

%prediction 预测
pre=@(x)(1+exp(-1*(t(1)+t(2)*x(1)+t(3)*x(2))))^-1;
t
end
