% 求解二阶ODE方程初值问题
% 其中未知函数z(t)的满足的ODE方程是 
%  z'' = (1-z^2)*z' -z 
% 初值条件是
%  z(0) = 2 
%  z'(0) = 0 
%  
% 我们首先把二阶ODE写成一阶ODEs
% 引入函数 y = [ z; z'] 
% 得到 y 的一阶ODEs
%  y' = [ y(2) ; (1-y(1)^2)*y(2)-y(1)] 
% 然后用ode45求解 上面的一阶级ODEs， 
% y(1) 就是所求的二阶ODE初值问题的解
clear, clc, close all; 
odefun = @(t,y) [ y(2); (1-y(1)^2)*y(2)-y(1)]; % 必须返回列向量
tspan = [0 20];
y0 = [2; 0];  % 这里看起来需要用列向量，但是实际上用行向量也能正确计算  
options = odeset('RelTol',1e-6,'AbsTol',1e-8);
[t,y] = ode45(odefun,tspan,y0,options); %计算出来的y有两列，分别对应函数和导数
plot(t,y(:,1),'-o',t,y(:,2),'-*');
legend('二阶ODE的解z', 'z''', 'Location', 'SouthEast')
xlabel('t')
title("用ode45求解二阶常微分方程的初值问题: z'' = (1-z^2)*z' -z  ; z(0)=2; z'(0)=0 ")
