function [ pv ] = pagerankEx1(A, alpha )
% 计算给定的网络中的网页的排名
%   A: 链接矩阵, A(i,j) = 0 表示从i到j没有链接，A(i,j)=1表示从i到j有一个链接
%   param alpha: 改造 t = alpha S + (1-alpha) e^T e/n,  
%  这里的s 是从A构造出来的转移矩阵, n = 网页个数，e = [1,1,...,1]^T.

if (~exist('alpha', 'var'))
    alpha = 0.85;
end

[m,n] = size(A) ;

if (m ~= n) 
	disp("链接矩阵需要是方阵");
	return 
end

for i=1:n
	for j=1:n
		if A(i,j) ~= 0 && A(i,j) ~= 1
			disp("错误:链接矩阵的元素的值既不是0也不是1, 而是");
			disp(sprintf("A(%d,%d)=%f\n",i,j,A(i,j)));
			return
		end
	end
end

%% 生成转移矩阵 S
S = A; 
%每个节点的对外链接数　
export_n  = sum(A,1); %A的第i列的和就是网页i的对外链接数
for i=1:n
	if export_n(i) > 0 
		S(:,i) = S(:,i)/export_n(i);
	else
		S(:,i) = 1/n;  
		%如果第i个节点的对外链接数是0，那么把转移矩阵的第i列
		%的元素设置为 1/n
end
%把矩阵改造为不可约的矩阵
E = 1/n * ones(n,n); 
S = alpha*S + (1-alpha)*E; 

%%求矩阵的模最大的特征值
%实际上可以证明，S的模最大的特征值是1
initial_v = ones(n,1);
it_max = 200;
tol = 1e-5;
%% 网页的得分
pv = power_method_Ex1(n, S, initial_v, it_max, tol); 

end
