% 北太天元模拟RLC 电路
% RLC 电路 包含 电容C、电感L,  和 电阻 R
% 在t =0 的时候打开开关， 电源是直流供电
%
% 常微分方程:
%  Vc '' (t)  + \frac{R}{L} Vc '(t) + \frac{1}{LC} Vc(t) = \frac{1}{LC} Vs(t)
%  这里的 Vs(t) = 5 
%
%  我们首先把二阶常微分方程写成一阶常微分方程组
%  引入 y(1) = Vc(t) ; y(2) = Vc'(t); 
%  d ( y(1) ) / dt  =  y(2);
%  d ( y(2) ) / dt = - R/L y(2)  - 1/(L*C)*y(1)  + 1/(L*C) *Vs

Vs = 5 ;  %直流电源的电压 
R = 1; %  电阻
C = 0.001; % 电容
L = 0.1;  % 电感 
tspan = [0 1]; % 计算的时间区间

y0 = [0; 0]; %初值

odefun = @(t,y) [ y(2);  -R/L*y(2)-1/(L*C)*y(1)+1/(L*C)*Vs];

options = odeset('RelTol', 1e-3, 'AbsTol', 1e-4);
[t, y] = ode45(odefun, tspan, y0, options);

plot(t, y(:,1), 'r-', 'LineWidth', 3);
xlabel("时间")
ylabel("Vc");
title("RLC 电路的响应Vc (也就是电容上的电压差)")
text(0.6,0.4, "北太天元数值计算通用软件",'FontSize',24)
