function  [x, k]=sor(A,b, omega,tol) 
% 松弛迭代法(SOR) 求解 Ax=b
% A: 矩阵
% b: 右端项
% omega: 松弛因子
% tol : 忍量
if( nargin ~= 4) 
	help sor
	return; 
end
if(isempty(A) || isempty(b) ) 
	error("矩阵A 或者右端项为空")
	return;
end

if( size(A,1) ~= size(A,2) )
	error("A 不是方阵");
	return;
end

if( ~isvector(b) )
	error("b 不是向量");
	return;
end

if( size(b,1) ~= size(A,2) )
	error("b 的行数 不等于 A 的列数");
	return;
end

if( ~isscalar(omega) || ~isscalar(tol))
	error("omega 和 tol 中至少有一个不是标量");
	return;
end

D=diag(diag(A));
L=D-tril(A);
U=D-triu(A);

x=zeros(size(b));

for   k=1:200
	x=(D-omega*L)\(omega*b+(1-omega)*D*x+omega*U*x);
	err=norm(b-A*x)/norm(b);
	if(err<tol)
        break;
	end
end	
end 
