%地固坐标系(x,y,z) --> 大地坐标系(BLH)

function position_BLH = xyz2BLH( position_xyz, rad )
	if(nargin == 1) 
		rad = false; 
	end

	if( nargin < 1 || length(position_xyz) ~= 3) 
		error("输入一个长度为3的向量(x,y,z)")
	end

	position_BLH = zeros(size(position_xyz)); 
	n = size(position_xyz,1);
	for i=1:n
		position_BLH(i,:) = xyz2BLH_single_approx(position_xyz(i,1),position_xyz(i,2),position_xyz(3), rad); 
	end
end


function degree = rad2degree(rad)
	degree = rad * 180/pi; 
end

function  BLH = xyz2BLH_single(x, y, z, rad)
		if nargin < 3 
			error("xyz2BLH_single(x,y,z,rad)  至少需要3个参数 )\n"); 
		end

		if nargin < 4
			rad = false;
		end

    a = 6378137.0000; %地球长轴半径
    b = 6356752.3141; %地球短轴半径 
    e2 = 1 - (b / a).^2;  
    p = sqrt(x.^2+y.^2);
    L = atan2(y,x);

		h_cal_N = @(B) a/sqrt(1-e2* (sin(B)).^2 ); 
		h_cal_H = @(N,B) p/cos(B) - N; 
		h_cal_B = @(N,H) atan( z / ((1-e2*N/(N+H))*p) ); 

		B = h_cal_B(1, 0);
		N = h_cal_N(B)
		H0 = 1e9;
		H = h_cal_H(N, B)
		while abs(H - H0) > 0.1
				B = h_cal_B(N, H)
				N = h_cal_N(B)
				H0 = H; 
				H = h_cal_H(N, B)
		end

    if rad
        BLH = [B, L, H];
    else
        BLH = [ rad2degree(B), rad2degree(L), H];
		end
end

/**
* 通过一些近似，解析的方式给出了一个近似解
*/
function  BLH = xyz2BLH_single_approx(x, y, z, rad)
		if nargin < 3 
			error("xyz2BLH_single(x,y,z,rad)  至少需要3个参数 )\n"); 
		end

		if nargin < 4
			rad = false;
		end

    a = 6378137.0000; %地球长轴半径
    b = 6356752.3141; %地球短轴半径 
    e2 = 1 - (b / a).^2;  
		e22 = (a/b).^2 - 1;  
    p = sqrt(x.^2+y.^2);
    L = atan2(y,x);

		theta = atan( z*a / ( p*b) ); 
		B = atan( (z+e22*b*(sin( theta ) )^3 ) / ( p - e2 * a *( cos(theta) )^3) ) ;
		N = a / sqrt( 1 - e2 * (sin(B))^2 ); 
		H =  p / cos(B) - N ; 

    if rad
        BLH = [B, L, H];
    else
        BLH = [ rad2degree(B), rad2degree(L), H];
		end
end
	
		

