% 翻转矩阵x 
% 如果x是向量，则把x的元素颠倒顺序重排
% 如果x是矩阵，则dim 可以取1或者2（默认值是1），分别表示
% 沿着列或者行翻转
% 例如 
% x=reshape(1:9,3,3)
% x = 
%    1.0000    4.0000    7.0000
%    2.0000    5.0000    8.0000
%    3.0000    6.0000    9.0000
% flip(x) 得到的结果是
%    3.0000    6.0000    9.0000
%    2.0000    5.0000    8.0000
%    1.0000    4.0000    7.0000
%flip(x,2)得到的结果是
%    7.0000    4.0000    1.0000
%    8.0000    5.0000    2.0000
%    9.0000    6.0000    3.0000

function y = flip (x, dim)

  if (nargin < 1 || nargin > 2)
    disp("参数个数只能是1或2")
    help flip 
    return; 
  end

  %sz 是一个1x2的矩阵，sz(1,1) 是行数，sz(1,2)是列数
  sz = size (x);
  %如果只有一个参数，要确定沿着哪个维度进行倒序
  if (nargin == 1)
    dim = find (sz > 1, 1);  %dim 获得第一个长度大于1的维 
  elseif ( ~(isscalar (dim) && dim == round(dim) ...
      && (dim == 1 || dim == 2) ) )
    disp("flip: 维数必须是1或者2");
    return 
  end

  %size(x,dim) 是获得第一个维上的长度
  idx_dim = size (x, dim):-1:1;
  if( dim == 1)
    y = x(idx_dim,:);
  else
    y = x(:,idx_dim);
  end
end


